'use strict';

var vscodeLanguageserverProtocol = require('vscode-languageserver-protocol');
var getPseudoClassCompletionItems = require('./get-pseudo-class-completion-items.cjs');
var getPseudoElementCompletionItems = require('./get-pseudo-element-completion-items.cjs');
var css = require('@master/css');
var common = require('../common.cjs');
var getMainCompletionItems = require('./get-main-completion-items.cjs');
var getValueCompletionItems = require('./get-value-completion-items.cjs');
var getQueryCompletionItems = require('./get-query-completion-items.cjs');
var getCssDataDocumentation = require('./get-css-data-documentation.cjs');

function querySyntaxCompletions(q = '', css$1 = new css.MasterCSS()) {
    const fields = q.split(' ');
    let field = fields[fields.length - 1];
    const triggerCharacter = q.charAt(q.length - 1);
    const invoked = triggerCharacter === ' ' || field.length === 0;
    if (invoked || field === common.GROUP_TRIGGER_CHARACTER || triggerCharacter === ';') {
        return getMainCompletionItems(css$1);
    }
    const isGroup = field.startsWith(common.GROUP_TRIGGER_CHARACTER);
    if (isGroup) {
        const declarationSeparatorMatches = Array.from(field.matchAll(/;(?=(?:[^'"]|'[^']*'|"[^"]*")*$)/g));
        if (declarationSeparatorMatches.length) {
            field = field.slice(declarationSeparatorMatches[declarationSeparatorMatches.length - 1].index + 1);
        } else {
            field = field.slice(1);
        }
    }
    const signs = css$1.Rules.filter(({ definition })=>definition.sign).map(({ definition })=>definition.sign);
    const keyMatch = field.match(new RegExp(`^[^${signs}'":\\s]+:`));
    const atMatches = Array.from(field.matchAll(/@(?=(?:[^'"]|'[^']*'|"[^"]*")*$)/g));
    const valueSeparatorMatch = field.match(/\|(?=(?:[^'"]|'[^']*'|"[^"]*")*$)/g);
    const atInvoked = atMatches.findIndex(({ index })=>index !== 0) !== -1;
    const firstColonIndex = keyMatch ? keyMatch[0].length - 1 : -1;
    const selectorInvokedRegex = new RegExp(`[${css.SELECTOR_SIGNS.join('')}](?=(?:[^'"]|'[^']*'|"[^"]*")*$)`);
    const key = keyMatch ? keyMatch[0].slice(0, firstColonIndex) : undefined;
    const styleNames = Object.keys(css$1.config.styles || {});
    const utilityNames = Object.keys(css$1.config.utilities || {});
    const isStyle = !!styleNames.find((eachStyleName)=>new RegExp(`^${eachStyleName}(?:\\b|_)`).test(field));
    const isUtility = !!utilityNames.find((eachUtilityName)=>new RegExp(`^${eachUtilityName}(?:\\b|_)`).test(field));
    // check by utilities and styles
    if (!isStyle && !isUtility) {
        if (key === undefined && !valueSeparatorMatch) {
            /**
             * The server capability sets '@' '~' as the trigger characters for at and adjacent selectors,
             * but these two characters are also the prefix symbols of `animation` and `transition`,
             * and should be filtered to prevent hints all completions items.
             * @example class="@"
             * @example class="~"
             */ if (field.startsWith(css.ANIMATION_SIGN) || field.startsWith(css.TRANSITION_SIGN)) {
                return getMainCompletionItems(css$1).filter((completionItem)=>completionItem.label.startsWith(field)).map((completionItem)=>({
                        ...completionItem,
                        label: completionItem.label.slice(1)
                    }));
            }
            return getMainCompletionItems(css$1);
        }
        if (!atInvoked && !selectorInvokedRegex.test(field.slice(firstColonIndex + 1))) {
            if (field.startsWith(css.ANIMATION_SIGN) && firstColonIndex === -1) {
                return getValueCompletionItems(css$1, 'animation');
            } else if (field.startsWith(css.TRANSITION_SIGN) && firstColonIndex === -1) {
                return getValueCompletionItems(css$1, 'transition');
            } else if (key && firstColonIndex !== -1) {
                return getValueCompletionItems(css$1, key);
            }
        }
    }
    /**
     * Not support conditional btn:hover and group
     */ if (isStyle || isGroup) return;
    if (atInvoked && [
        css.AT_SIGN,
        ...css.QUERY_COMPARISON_OPERATORS,
        ...css.QUERY_LOGICAL_OPERATORS
    ].includes(triggerCharacter)) {
        return getQueryCompletionItems(css$1, triggerCharacter, field);
    }
    if (!atInvoked && common.SELECTOR_TRIGGER_CHARACTERS.includes(triggerCharacter)) {
        const pseudoElementCompletionItems = getPseudoElementCompletionItems(css$1, field);
        if (field.endsWith('::')) {
            /**
             * Consider trigger characters and fix insertText.
             * @incorrect class="text:center::" -> class="text:center::::after"
             * @correct class="text:center::" -> class="text:center::after"
             */ pseudoElementCompletionItems.forEach((completionItem)=>completionItem.insertText = completionItem.label.slice(2));
            return pseudoElementCompletionItems;
        } else {
            const completionItems = getPseudoClassCompletionItems(css$1, field);
            completionItems.push(...pseudoElementCompletionItems);
            /**
             * Consider trigger characters and fix insertText.
             * @incorrect class="text:center:" -> class="text:center::active"
             * @correct class="text:center:" -> class="text:center:active"
             */ if (triggerCharacter === ':') {
                completionItems.forEach((completionItem)=>completionItem.insertText = completionItem.label.slice(1));
            } else {
                completionItems.push({
                    label: '.<class>',
                    insertText: '.',
                    documentation: getCssDataDocumentation.getCSSDataDocumentation(undefined, {
                        generatedCSS: css.generateCSS([
                            field + '.class'
                        ], css$1),
                        docs: 'selectors'
                    }),
                    kind: vscodeLanguageserverProtocol.CompletionItemKind.Class
                });
            }
            return completionItems;
        }
    }
}

module.exports = querySyntaxCompletions;
